---------------------------------------------------
---------------------------------------------------
--  Easy 1 Processor Architecture				 --
--  University of Puerto Rico					 --
--  Mayaguez Campus								 --
--	Department of Computer Engineering			 --
-- 												 --
--	Copyright (c) 2003							 --
--	All rigths reserved.						 --
--										   		 --
--	Prof. Bienvenido Vlez						 --
--	Abdiel Avils								 --
--	Rafael S. Garca							 --
--	Nesmary Hernndez 							 --
-- 	Carlos Prez								 --
---------------------------------------------------
---------------------------------------------------
-- 	[easy1_data_paths.vhd]
--	Easy 1 Data Paths
--	This module describes the Easy 1 Data Paths.
--	The Easy 1 data paths connects the different
--	entities that compose the Easy 1 processor. 
---------------------------------------------------	
---------------------------------------------------
--	Libraries declarations
library ieee;
use ieee.std_logic_1164.all;   
use easy1_package.all;
--
---------------------------------------------------
--	Entity declaration
--	@reset     		- reset signal
--	@clock  	   	- clock source
--	@alu_op			- control signal for the ALU
--	@pc_sel   		- control signal for the PC
--	@ir_le  	   	- load signal for the IR
--	@pc_is	     	- control signal for the PC
--	@pc_le 			- load signal for the PC
--	@di_le  	  	- load signal for the DI
--	@ac_le	     	- load signal for the AC
--	@ac_cl     		- clear signal for the AC
--	@a0_sel			- control signal for the A0
--	@a0_le	    	- load signal for the A0
--	@sxt_sel   		- control signal for the SXTL
--	@data_in	   	- data input Bus
--	@opcode			- opcode signal	for the CU
--	@i      	  	- indirect bit (bit 15 IR)
--	@ac_15	     	- sign bit (bit 15 AC)
--	@data_out		- data ouput Bus
--	@address_bus	- address Bus
--
entity easy1_data_paths is
	
	port (
		reset 	 	: in std_logic:='0';
		clock 	 	: in std_logic:='0';
		alu_op      : in std_logic_vector(2 downto 0):="000";
		pc_sel      : in std_logic_vector(1 downto 0):="00";
		ir_le       : in std_logic:='0';
		pc_is       : in std_logic:='0';
		pc_le	 	: in std_logic:='0';
		di_le       : in std_logic:='0';
		ac_le       : in std_logic:='0';
		ac_cl		: in std_logic:='0';
		a0_sel      : in std_logic:='0';
		a0_le       : in std_logic:='0';
		sxt_sel     : in std_logic_vector(1 downto 0):="00";
		data_in		: in std_logic_vector(15 downto 0):=X"0000";
		opcode      : out std_logic_vector(4 downto 0):="00000";
		i           : out std_logic:='0';
		ac_15       : out std_logic:='0';
		data_out	: out std_logic_vector(15 downto 0):=X"0000";
		address_bus	: out std_logic_vector(15 downto 0):=X"0000"
		);														
		
end easy1_data_paths;
--
----------------------------------------------------
--	Architecture declaration
--	Signals :
--		@AA_bus			- signal that connects the A Bus to
--						  the ALU a0, the PC intput, the A0
--						  input, and SXTL module output
--		@PC_out_bus		- signal that connects the PC output 
--						  to the A0 input	
--		@AC_out_bus		- signal that connects the Data Bus to
--						  the AC output and the ALU b0
--		@ALU_out_bus	- signal that connects the ALU output 
--						  to the AC input	
--		@DI_out_bus		- signal that connects the DI output 
--						  to the SXTL input	
--		@IR_out			- signal that connects the IR output 
--						  to the signals to be connected to
--						  the Control Unit
--		@A0_out_bus		- signal that connects the A0 output 
--						  to the Address Bus	   
--
--	Components :
--		@easy1_alu		- Easy 1 ALU
--		@easy1_pc 		- Easy 1 Program Counter
--		@easy1_a0 		- Easy 1 Address Register
--		@easy1_reg_ms	- Easy 1 Master/Slave Register
--		@easy1_SXTL		- Easy 1 Sign Extend Logic
--
architecture arq of easy1_data_paths is	 
	
	component easy1_alu
		port(
			a0      : in  std_logic_vector(15 downto 0):=X"0000";
			b0      : in  std_logic_vector(15 downto 0):=X"0000";
			alu_op  : in  std_logic_vector( 2 downto 0):="000";
			ac_alu  : out std_logic_vector(15 downto 0):=X"0000"
			);
	end component;
	
	component easy1_pc
		port(
			clock	: in std_logic:='0';
			reset	: in std_logic:='0';
			pc_is	: in std_logic:='0';
			pc_sel	: in std_logic_vector(1 downto 0);
			pc_le	: in std_logic:='0';
			a_bus	: in std_logic_vector(15 downto 0):=X"0000";
			pc_out	: inout std_logic_vector(15 downto 0):=X"0000"
			);
	end component; 
	
	component easy1_a0
		port( 
			clock	: in std_logic:='0';
			a0_sel	: in std_logic:='0';	
			reset   : in std_logic:='0';
			a0_le	: in std_logic:='0';
			pc_bus	: in std_logic_vector(15 downto 0):=X"0000";
			a_bus	: in std_logic_vector(15 downto 0):=X"0000";
			a0_out	: out std_logic_vector(15 downto 0):=X"0000"
			);	
	end component;

	component easy1_reg_ms
		port(
			clock	: 	in std_logic:='0';
			input	:	in std_logic_vector(15 downto 0):=X"0000";
			load	:	in std_logic:='0';
			clear	:	in std_logic:='0';
			o		:	out std_logic_vector(15 downto 0):=X"0000"
			);
	end component;
	
	component easy1_SXTL
		port(
			sxt_sel		: in std_logic_vector(1 downto 0):="00";
			input_SXT	: in std_logic_vector(15 downto 0):=X"0000";
			output_SXT	: out std_logic_vector(15 downto 0):=X"0000"
			);
	end component;
	
	signal AA_bus 		: std_logic_vector(15 downto 0):=X"0000";  
	signal PC_out_bus	: std_logic_vector(15 downto 0):=X"0000";
	signal AC_out_bus	: std_logic_vector(15 downto 0):=X"0000";
	signal ALU_out_bus	: std_logic_vector(15 downto 0):=X"0000";
	signal DI_out_bus	: std_logic_vector(15 downto 0):=X"0000";
	signal IR_out		: std_logic_vector(15 downto 0):=X"0000";
	signal A0_out_bus	: std_logic_vector(15 downto 0):=X"0000";  	
	
begin
	--	Port mapping:
	--		Component instances are mapped to internal signals,
	--		inputs, and outputs.
	--
	-- 	Arithmetic and Logic Unit
	ALU: easy1_alu
	port map(
		a0 => AA_bus,
		b0 => AC_out_bus,
		alu_op => alu_op,
		ac_alu => ALU_out_bus
		);			  
		
	-- 	Program Counter
	PC1: easy1_pc
	port map(
		clock => clock,	
		reset => reset,
		pc_is => pc_is,
		pc_sel => pc_sel,	 
		pc_le => pc_le,
		a_bus => AA_bus,
		pc_out => PC_out_bus
		);
		
	-- 	Address Register
	A01: easy1_a0
	port map(	 
		clock => clock,
		a0_sel => a0_sel, 
		reset => reset,
		a0_le => a0_le,
		pc_bus => PC_out_bus,
		a_bus => AA_bus,
		a0_out => A0_out_bus
		);
		
	-- 	Sign Extend Logic
	SXTL: easy1_SXTL
	port map(
		sxt_sel => sxt_sel,
		input_SXT => DI_out_bus,
		output_SXT => AA_bus
		);
		
	-- 	Accumulator
	AC1: easy1_reg_ms
	port map(
		clock => clock,
		input => ALU_out_bus,
		load => ac_le,
		clear => ac_cl,
		o => AC_out_bus
		);
		
	--	Instruction Register
	IR1: easy1_reg_ms
	port map(
		clock => clock,
		input => data_in,
		load => ir_le,
		clear => '0',
		o => IR_out	
		);
		
	-- 	Data Input Register
	DI1: easy1_reg_ms
	port map(
		clock => clock,
		input => data_in,
		load => di_le,
		clear => reset,
		o => DI_out_bus	
		);
	
	--	Opcode
	opcode <= IR_out(14 downto 10);
	--	Indirect bit
	i <= IR_out(15);
	--	Sign bit
	ac_15 <= AC_out_bus(15);
	--	Address Bus
	address_bus <= A0_out_bus;
	--	Data Bus
	data_out <= AC_out_bus;
	
end arq;
----------------------------------------------------
----------------------------------------------------